@@ -6,7 +6,7 @@
  */
 
 /*
-    Copyright 2011-2017 Thibaut Paumard, Frederic Vincent
+    Copyright 2011-2019 Thibaut Paumard, Frederic Vincent
 
     This file is part of Gyoto.
 
@@ -202,6 +202,17 @@ class Gyoto::Astrobj::Generic
 
   // Data : 
   // -----
+private:
+
+  /**
+   * \brief Whether some virtual methods are implemented
+   *
+   * The default implementations of some methods call
+   * one-another. This member is used internally to avoid infinite
+   * recursion.
+   */
+  int __defaultfeatures;
+
  protected:
 
   /**
@@ -221,6 +232,14 @@ class Gyoto::Astrobj::Generic
    */                                         
   double rmax_; ///< Maximum distance to the center of the coordinate system [geometrical units]
 
+  /**
+   * Maximum step for Photon integation when inside the sphere of
+   * radius #rmax_.
+   *
+   * #deltamaxinsidermax_ is in geometrical units.
+   */
+  double deltamaxinsidermax_; ///< Maximum Photon integration step inside #rmax_ [geometrical units]
+
   bool flag_radtransf_; ///< 1 if radiative transfer inside Astrobj, else 0
 
   int shadow_; ///< 1 to highlight the shadow region in the image
@@ -332,6 +351,8 @@ class Gyoto::Astrobj::Generic
    */
   virtual void rMax(double val, std::string const &unit); ///< Set maximal distance from center of coordinate system
 
+  GYOTO_OBJECT_ACCESSORS_UNIT(deltaMaxInsideRMax);
+
   /**
    * Set flag indicating that radiative transfer should be integrated,
    * i.e. the object is to be considered optically thin.
@@ -443,9 +464,12 @@ class Gyoto::Astrobj::Generic
    * You can overload it for your Astrobj. The generic implementation
    * calls emission(), integrateEmission() and transmission() below.
    */
+  virtual void processHitQuantities(Photon * ph, state_t const &coord_ph_hit,
+                                   double const * coord_obj_hit, double dt,
+                                   Astrobj::Properties* data) const;
   virtual void processHitQuantities(Photon* ph, double* coord_ph_hit,
                                    double* coord_obj_hit, double dt,
-                                   Astrobj::Properties* data) const;
+                                   Astrobj::Properties* data) const = delete ;
 
   /**
    * \brief Specific intensity I<SUB>&nu;</SUB>
@@ -496,9 +520,12 @@ class Gyoto::Astrobj::Generic
    * \param coord_ph Photon coordinate
    * \param coord_obj Emitter coordinate at current photon position
    */
+  virtual double emission(double nu_em, double dsem, state_t const &coord_ph,
+			  double const coord_obj[8]=NULL)
+    const ;
   virtual double emission(double nu_em, double dsem, double coord_ph[8],
 			  double coord_obj[8]=NULL)
-    const ;
+    const = delete; ///< Obsolete, update your code;
 
   /**
    * \brief Specific intensity I<SUB>&nu;</SUB> for several values of &nu;<SUB>em</SUB>
@@ -522,17 +549,31 @@ class Gyoto::Astrobj::Generic
    * \param coord_obj Emitter coordinate at current photon position
    * \return I<SUB>&nu;</SUB> or dI<SUB>&nu;</SUB> [W m-2 sr-2]
    */
+  virtual void emission(double Inu[], double const nu_em[], size_t nbnu,
+			double dsem, state_t const &coord_ph,
+			double const coord_obj[8]=NULL) const ;
   virtual void emission(double Inu[], double nu_em[], size_t nbnu,
 			double dsem, double coord_ph[8],
-			double coord_obj[8]=NULL) const ; 
+			double coord_obj[8]=NULL) const = delete ; ///< Obsolete, update your code
 
   /**
    * \brief emission and transmission together
    */
+  virtual void radiativeQ(double Inu[], double Taunu[], 
+			  double const nu_em[], size_t nbnu,
+			  double dsem, state_t const &coord_ph,
+			  double const coord_obj[8]=NULL) const ;
   virtual void radiativeQ(double Inu[], double Taunu[], 
 			  double nu_em[], size_t nbnu,
 			  double dsem, double coord_ph[8],
-			  double coord_obj[8]=NULL) const ; 
+			  double coord_obj[8]=NULL) const = delete ; 
+  virtual void radiativeQ(double *Inu, double *Qnu, double *Unu, double *Vnu,
+			  double *alphaInu, double *alphaQnu,
+			  double *alphaUnu, double *alphaVnu,
+			  double *rQnu, double *rUnu, double *rVnu,
+			  double const *nuem , size_t nbnu, double dsem,
+			  state_t const &cph,
+			  double const *co) const ;
 
   /**
    * Compute the integral of emission() from &nu;<SUB>1</SUB> to
@@ -546,8 +587,10 @@ class Gyoto::Astrobj::Generic
    * instrument.
    */
   virtual double integrateEmission(double nu1, double nu2, double dsem,
-                                  double c_ph[8], double c_obj[8]=NULL) const;
+				   state_t const &c_ph, double const c_obj[8]=NULL) const;
     ///< &int;<SUB>&nu;<SUB>1</SUB></SUB><SUP>&nu;<SUB>2</SUB></SUP> I<SUB>&nu;</SUB> d&nu; (or j<SUB>&nu;</SUB>)
+  virtual double integrateEmission(double nu1, double nu2, double dsem,
+				   double c_ph[8], double c_obj[8]=NULL) const=delete; ///< Obsolete, update your code
 
   /**
    * Like double integrateEmission(double nu1, double nu2, double
@@ -556,8 +599,12 @@ class Gyoto::Astrobj::Generic
    */
   virtual void integrateEmission(double * I, double const * boundaries,
 				 size_t const * chaninds, size_t nbnu,
-				 double dsem, double *cph, double *co) const;
+				 double dsem, state_t const &cph, double const *co) const;
     ///< &int;<SUB>&nu;<SUB>1</SUB></SUB><SUP>&nu;<SUB>2</SUB></SUP> I<SUB>&nu;</SUB> d&nu; (or j<SUB>&nu;</SUB>)
+  virtual void integrateEmission(double * I, double const * boundaries,
+				 size_t const * chaninds, size_t nbnu,
+				 double dsem, double *cph, double *co) const = delete;
+  ///< Obsolete, update your code
 
   /**
    * transmission() computes the transmission of this fluid element or
@@ -568,8 +615,11 @@ class Gyoto::Astrobj::Generic
    * \param coord Photon coordinate
    * \param dsem geometrical length in geometrical units
    */
-  virtual double transmission(double nuem, double dsem, double coord[8]) const ;
+  virtual double transmission(double nuem, double dsem, state_t const &coord_ph, double const coord_obj[8]) const ;
      ///< Transmission: exp( &alpha;<SUB>&nu;</SUB> * ds<SUB>em</SUB> )
+  virtual double transmission(double nuem, double dsem, state_t const &coord) const = delete;  ///< Obsolete, update your code
+  virtual double transmission(double nuem, double dsem, double coord[8]) const = delete;
+  ///< Obsolete, update your code
 
 };
 
@@ -637,6 +687,10 @@ class Gyoto::Astrobj::Properties : public Gyoto::SmartPointee {
    */
   double *spectrum; ///< GYOTO_QUANTITY_SPECTRUM    : Spectrum
 
+  double *stokesQ;
+  double *stokesU;
+  double *stokesV;
+
   /**
    *  I<SUB>&nu;<SUB>1</SUB></SUB><SUP>&nu;<SUB>2</SUB></SUP>, the
    *  integral of I<SUB>&nu;</SUB> over each spectral channel
