@@ -3,7 +3,7 @@
  * \brief Gyoto ubiquitous macros and typedefs
  */
 /*
-    Copyright 2011 Thibaut Paumard
+    Copyright 2011, 2018 Thibaut Paumard
 
     This file is part of Gyoto.
 
@@ -26,6 +26,7 @@
 
 #include "GyotoConfig.h"
 #include <float.h>
+#include <vector>
 
 /**
  * \brief Replacement for GNU extension sincos
@@ -46,6 +47,8 @@
 
 /* Typedef for various Gyoto data types */
 namespace Gyoto {
+  typedef std::vector<double> state_t;
+
   //\{
   /**
    * \name Observable quantities
@@ -95,7 +98,13 @@ namespace Gyoto {
    */ 
 #define GYOTO_QUANTITY_IMPACTCOORDS  32
   /// Spectrum: I<SUB>&nu;</SUB> at each frequency in Scenery::screen_->getMidpoints().
-#define GYOTO_QUANTITY_SPECTRUM     512
+#define GYOTO_QUANTITY_SPECTRUM      64
+  /// SpectrumStokesQ
+#define GYOTO_QUANTITY_SPECTRUM_STOKES_Q     128
+  /// SpectrumStokesU
+#define GYOTO_QUANTITY_SPECTRUM_STOKES_U     256
+  /// SpectrumStokesV
+#define GYOTO_QUANTITY_SPECTRUM_STOKES_V     512
   /// Spectrum: &int;<SUB>&nu;<SUB>1</SUB></SUB><SUP>&nu;<SUB>2</SUB></SUP>I<SUB>&nu;</SUB> d&nu; in each frequency channel in Scenery::screen_.
 #define GYOTO_QUANTITY_BINSPECTRUM 1024
   /* Astrobj-specific */
@@ -109,6 +118,10 @@ namespace Gyoto {
 #define GYOTO_QUANTITY_USER4         4096
   /// User5: Gyoto::Astrobj specific Gyoto::Quantity_t
 #define GYOTO_QUANTITY_USER5         2048
+  /// Bitwise or of all spectral quantities
+#define GYOTO_QUANTITY_SPECTRAL (GYOTO_QUANTITY_SPECTRUM | GYOTO_QUANTITY_SPECTRUM_STOKES_Q | GYOTO_QUANTITY_SPECTRUM_STOKES_U | GYOTO_QUANTITY_SPECTRUM_STOKES_V | GYOTO_QUANTITY_BINSPECTRUM)
+  /// Bitwise or of all the Stokes parameters
+#define GYOTO_QUANTITY_SPECTRUM_STOKES (GYOTO_QUANTITY_SPECTRUM | GYOTO_QUANTITY_SPECTRUM_STOKES_Q | GYOTO_QUANTITY_SPECTRUM_STOKES_U | GYOTO_QUANTITY_SPECTRUM_STOKES_V)
   //\}
 
   /**
@@ -264,6 +277,20 @@ namespace Gyoto {
    */
 #define GYOTO_DEBUG_EXPR(a) GYOTO_DEBUG << #a << "=" << a << std::endl
 
+  /// Output expression value unconditionally
+  /**
+   * Output both code and value that code yield. For instance:
+   * \code
+   * int a=1, b=2;
+   * GYOTO_DEBUG_THIS_EXPR(a+b);
+   * \endcode
+   * will essentially output:
+   * \code
+   * DEBUG: <function signature>: a+b=3
+   * \endcode
+   */
+#define GYOTO_DEBUG_THIS_EXPR(a) GYOTO_DEBUG_THIS << #a << "=" << a << std::endl
+
   /// Output array content in debug mode
   /**
    * Output, only in debug, name and content of array.
@@ -288,9 +315,20 @@ namespace Gyoto {
       std::cerr << "," << a[_gyoto_debug_array_i] ;			\
     std::cerr << "]" << std::endl ;}
 
-  /// Display debug message
+  /// Display debug message (unconditionally)
   /**
-   * Message is deiplayed only if #GYOTO_DEBUG_MODE is true and is
+   * Message is prepended with the word "DEBUG:" and the signature of
+   * the function.
+   *
+   * \code
+   * GYOTO_DEBUG << "message" << endl;
+   * \endcode
+   */
+#define GYOTO_DEBUG_THIS std::cerr << "DEBUG: " << __PRETTY_FUNCTION__ << ": "
+
+  /// Display debug message (in debug mode)
+  /**
+   * Message is displayed only if #GYOTO_DEBUG_MODE is true and is
    * prepended with the word "DEBUG:" and the signature of the
    * function.
    *
@@ -298,7 +336,7 @@ namespace Gyoto {
    * GYOTO_DEBUG << "message" << endl;
    * \endcode
    */
-#define GYOTO_DEBUG  if (GYOTO_DEBUG_MODE) std::cerr << "DEBUG: " << __PRETTY_FUNCTION__ << ": "
+#define GYOTO_DEBUG  if (GYOTO_DEBUG_MODE) GYOTO_DEBUG_THIS
 
   /// Start debug-only block.
   /**
@@ -367,7 +405,7 @@ namespace Gyoto {
 /**
  * \brief Default value for delta_max_over_r_
  *
- * For invastigations close to the event horizon, 0.5 is usually
+ * For investigations close to the event horizon, 0.5 is usually
  * fine. If high accuracy is needed long after deflection (weak
  * lensing), then this must be smaller. A good test is to look at a
  * MinDistance map for a FixedStar: it must be smooth.
@@ -387,7 +425,7 @@ namespace Gyoto {
  *
  * E.g. in Gyoto::Photon::findMin(), Gyoto::Photon::findValue().
  */
-#define GYOTO_T_TOL 1e-4
+#define GYOTO_T_TOL 1e-4 //1e-7 //1e-4
 
 #define GYOTO_KERR_HORIZON_SECURITY 0.01
 
@@ -420,18 +458,24 @@ namespace Gyoto {
 #define GYOTO_C          299792458.
 /// \brief Celerity of light (cm/s)
 #define GYOTO_C_CGS 2.99792458e10
+/// \brief Celerity of light squared c^2 in cgs
+#define GYOTO_C2_CGS 8.98755178736817668096e+20
+/// \brief Inverse of celerity of light squared in cgs
+#define GYOTO_C2_CGS_M1 1.1126500560536184087938986e-21
 /// \brief Gravitational constant (SI = m^3 * kg^-1 * s-2)
 #define GYOTO_G 6.67428e-11
 /// \brief Gravitational constant (cgs: cm^3 * g^-1 * s-2)
 #define GYOTO_G_CGS 6.67428e-8
 /// \brief G/c^2=6.67428e-11/299792458.^2
-#define GYOTO_G_OVER_C_SQUARE 7.426138e-28
+#define GYOTO_G_OVER_C_SQUARE 7.4261380161175445989e-28
+/// \brief G/c^2 in cgs units
+#define GYOTO_G_OVER_C_SQUARE_CGS 7.4261380161175445989e-29
 /// \brief Planck's constant (h) in SI (J.s=kg.m^2/s) 
 #define GYOTO_PLANCK 6.62606896e-34
 /// \brief Planck's constant (h) in c.g.s (g.cm^2/s) 
 #define GYOTO_PLANCK_CGS 6.62606896e-27
 /// \brief h/c^2 in SI (kg.s)
-#define GYOTO_PLANCK_OVER_C_SQUARE 7.372496e-51
+#define GYOTO_PLANCK_OVER_C_SQUARE 7.3724959997591407964e-51
 /// \brief Boltzmann's constant (k) in SI (J/K)
 #define GYOTO_BOLTZMANN 1.3806504e-23
 /// \brief Boltzmann's constant (k) in cgs (erg/K)
@@ -439,7 +483,7 @@ namespace Gyoto {
 /// \brief Stefan-Boltzmann's constant (sigma) in cgs (erg/cm2/s/K4)
 #define GYOTO_STEFANBOLTZMANN_CGS 5.670373e-5
 /// \brief h/k (K.s = K/Hz)
-#define GYOTO_PLANCK_OVER_BOLTZMANN 4.7992373e-11
+#define GYOTO_PLANCK_OVER_BOLTZMANN 4.7992373449498869688e-11
 /// \brief ideal gas constant R in SI
 #define GYOTO_GAS_CST 8.3144621
 /// \brief ideal gas constant R in erg/(K mol)
@@ -464,6 +508,10 @@ namespace Gyoto {
 #define GYOTO_ATOMIC_MASS_UNIT_CGS 1.660537781e-24
 /// \brief Factor to convert I<SUB>&nu;</SUB> from c.g.s. to SI
 #define GYOTO_INU_CGS_TO_SI 0.001
+/// \brief Factor to convert J<SUB>&nu;</SUB> from c.g.s. to SI
+#define GYOTO_JNU_CGS_TO_SI (GYOTO_INU_CGS_TO_SI  * 100.)
+/// \brief Factor to convert alpha<SUB>&nu;</SUB> from c.g.s. to SI
+#define GYOTO_ANU_CGS_TO_SI 100.
 
 /// \brief Sun mass (kg)
 #define GYOTO_SUN_MASS    1.98843e30
@@ -496,6 +544,30 @@ namespace Gyoto {
 
 //\}
 
+//\{
+/**
+ *\name Observer kind
+ */
+/// \brief Type for observer kind
+#define obskind_t unsigned int
+/// \brief ObserverAtInfinity
+#define GYOTO_OBSKIND_ATINFINITY         0
+/// \brief KeplerianObserver
+#define GYOTO_OBSKIND_KEPLERIAN          1
+/// \brief ZAMO
+#define GYOTO_OBSKIND_ZAMO               2
+/// \brief VelocitySpecified
+#define GYOTO_OBSKIND_VELOCITYSPECIFIED  3
+/// \brief FullySpecified
+#define GYOTO_OBSKIND_FULLYSPECIFIED     4
+//\}
+
+/// \brief Stringify macro content
+#define GYOTO_STRINGIFY(a) GYOTO_STRINGIFY_ARGUMENT(a)
+
+/// \brief Stringify argument
+#define GYOTO_STRINGIFY_ARGUMENT(a) #a
+
 #ifndef GYOTO_NO_DEPRECATED
 #warning Using deprecated method names.\
   Define GYOTO_NO_DEPRECATED to disable.
@@ -546,10 +618,10 @@ namespace Gyoto {
 # define getDelta           delta
 # define setDelta           delta
 # define getDelta           delta
-# define setDelta0          delta0
-# define getDelta0          delta0
-# define setAlpha0          alpha0
-# define getAlpha0          alpha0
+# define setDangle2         dangle2
+# define getDangle2         dangle2
+# define setDangle1         dangle1
+# define getDangle1         dangle1
 # define setAnglekind       anglekind
 # define getTmin            tMin
 # define setTmin            tMin
