@@ -102,9 +102,16 @@ class Gyoto::Astrobj::Disk3D : public Gyoto::Astrobj::Generic {
 
   int zsym_; ///< 1 to symmetrize the grid z -> -z (default case)
 
+  double tPattern_; ///< If the disk is being rotated (like a pattern disk) this is the origin of time for this rotation
+  double omegaPattern_; ///< If the disk is being rotated (like a pattern disk) this is the rotation velocity dphi/dt
+
   // Constructors - Destructor
   // -------------------------
  public:
+  GYOTO_OBJECT;
+  // fillProperty is overridden to remove leading "!" from FITS filename
+  void fillProperty(Gyoto::FactoryMessenger *fmp, Property const &p) const;
+
   Disk3D(); ///< Standard constructor.
   
   Disk3D(const Disk3D& ) ;///< Copy constructor.
@@ -124,6 +131,16 @@ class Gyoto::Astrobj::Disk3D : public Gyoto::Astrobj::Generic {
   virtual void fitsWrite(std::string filename_);
 #endif
 
+  void file(std::string const &f);
+  std::string file() const;
+  void zsym(bool t);
+  bool zsym() const;
+  void tPattern(double t);
+  double tPattern() const;
+  void omegaPattern(double t);
+  double omegaPattern() const;
+
+
   /// Set Disk3D::emissquant_.
   /**
    * The pointer is copied directly, not the array content.
@@ -240,10 +257,6 @@ class Gyoto::Astrobj::Disk3D : public Gyoto::Astrobj::Generic {
   /// Get Disk3D::phimax_.
   double phimax() const;
 
-  virtual int setParameter(std::string name,
-			   std::string content,
-			   std::string unit);
-
  protected:
   void getIndices(size_t i[4], double const co[4], double nu=0.) const ;
   ///< Get emissquant_ cell corresponding to position co[4].
@@ -261,12 +274,6 @@ class Gyoto::Astrobj::Disk3D : public Gyoto::Astrobj::Generic {
    */
   virtual void getVelocity(double const pos[4], double vel[4])  ;
 
- public:
-#ifdef GYOTO_USE_XERCES
-  virtual void fillElement(FactoryMessenger *fmp) const ;
-  virtual void setParameters(FactoryMessenger *fmp);
-#endif
-
 };
 
 #endif
