@@ -5,7 +5,7 @@
  */
 
 /*
-    Copyright 2011 Frederic Vincent, Thibaut Paumard
+    Copyright 2011, 2018 Frederic Vincent, Thibaut Paumard
 
     This file is part of Gyoto.
 
@@ -118,6 +118,8 @@ class Gyoto::Metric::KerrBL : public Metric::Generic {
   virtual void circularVelocity(double const pos[4], double vel [4],
 				double dir=1.) const ;
 
+  virtual void zamoVelocity(double const pos[4], double vel[4]) const ;
+
  public:
   virtual void MakeCoord(const double coordin[8], const double cst[5], double coordout[8]) const ;
   ///< Inverse function of MakeMomentumAndCst
@@ -133,11 +135,11 @@ class Gyoto::Metric::KerrBL : public Metric::Generic {
      cst=[a,E,L,Q,1/Q],dy/dtau=F(y,cst), h=proper time step. 
      For KerrBL geodesic computation.
    */
-  int myrk4(Worldline * line, const double coordin[8], double h, double res[8]) const; //external-use RK4
+  int myrk4(Worldline * line, Gyoto::state_t const &coordin, double h, Gyoto::state_t &res) const; //external-use RK4
   
  public:
   int myrk4(const double coor[8], const double cst[5], double h, double res[8]) const;///< Internal-use RK4 proxy
-  int myrk4_adaptive(Gyoto::Worldline* line, const double coor[8], double lastnorm, double normref, double coor1[8], double h0, double& h1, double h1max=GYOTO_DEFAULT_DELTA_MAX) const; ///< Internal-use adaptive RK4 proxy
+  int myrk4_adaptive(Gyoto::Worldline* line, Gyoto::state_t const &coor, double lastnorm, double normref, Gyoto::state_t &coor1, double h0, double& h1, double h1max=GYOTO_DEFAULT_DELTA_MAX) const; ///< Internal-use adaptive RK4 proxy
   /**
    * \brief Ensure conservation of the constants of motion
    *
@@ -170,10 +172,10 @@ class Gyoto::Metric::KerrBL : public Metric::Generic {
   void setParticleProperties(Worldline* line, const double* coord) const;
   virtual int isStopCondition(double const * const coord) const;
   
-  void observerTetrad(std::string const obskind,
-		      double const pos[4], double fourvel[4],
-		      double screen1[4], double screen2[4], 
-		      double screen3[4]) const;
+  virtual void observerTetrad(double const pos[4], double fourvel[4],
+			      double screen1[4], double screen2[4],
+			      double screen3[4]) const ;
+
 };
 
 #endif
