@@ -34,6 +34,7 @@
 #include <iostream>
 #include <fstream>
 #include <string>
+#include <vector>
 
 #include <GyotoSmartPointer.h>
 #include <GyotoObject.h>
@@ -44,6 +45,7 @@
 
 namespace Gyoto {
   namespace Metric {
+
     class Generic;
 
     /// A function to build instances of a specific Metric::Generic sub-class
@@ -336,6 +338,19 @@ class Gyoto::Metric::Generic
   virtual void circularVelocity(double const pos[4], double vel[4],
 				double dir=1.) const ;
 
+  /**
+   * \brief Yield ZAMO velocity at a given position
+   *
+   * Give the velocity of a zero angular momentul observer (whatever
+   * is closest to "at rest"). The default implementation simply
+   * projects (1, 0, 0, 0) othogonally along ephi and normalizes it,
+   * thus ensuring that vel is orthogonal to ephi.
+   *
+   * \param pos input: position,
+   * \param vel output: velocity,
+   */
+  virtual void zamoVelocity(double const pos[4], double vel[4]) const ;
+
   /**
    * Set coord[4] so that the 4-velocity coord[4:7] is lightlike,
    * i.e. of norm 0. There may be up to two solutions. coord[4] is set
@@ -362,6 +377,30 @@ class Gyoto::Metric::Generic
   virtual void nullifyCoord(double coord[8], double& tdot2) const;
   ///< Set tdot (coord[4]) such that coord is light-like and return other possible tdot
 
+  /**
+   * \brief Normalize fourvelvel to -1
+   *
+   * First computes threevel as xiprime=xidot/x0dot for i in {1, 2,
+   * 3}, then computes x0dot using SyPrimeToTdot, then computes again
+   * xidot as xidot=xiprime*x0dot.
+   *
+   * \param[in,out] coord 8-position, coord[4-7] will be set according
+   * to the other elements;
+   */
+  virtual void normalizeFourVel(double coord[8]) const;
+
+  /**
+   * \brief Normalize fourvelvel to -1
+   *
+   * First computes threevel as xiprime=xidot/x0dot for i in {1, 2,
+   * 3}, then computes x0dot using SyPrimeToTdot, then computes again
+   * xidot as xidot=xiprime*x0dot.
+   *
+   * \param[in] pos 4-position;
+   * \param[in,out] fourvel 4-velocity, will be renormalized.
+   */
+  virtual void normalizeFourVel(double const pos[4], double fourvel[4]) const;
+
 
   /**
    * Compute the scalarproduct of the two quadrivectors u1 and u2 in
@@ -374,10 +413,37 @@ class Gyoto::Metric::Generic
   virtual double ScalarProd(const double pos[4],
 		    const double u1[4], const double u2[4]) const; ///< Scalar product
 
+  /**
+   * Compute the norm of the quadrivector u1 in
+   * this Metric, at point pos expressed in coordinate system sys.
+   * \param[in] pos 4-position;
+   * \param[in] u1 quadrivector;
+   * \return ||u1||
+   */
+  double norm(const double pos[4],
+		      const double u1[4]) const; ///< Scalar product
+
+  ///\brief multiply vector by scalar
+  void multiplyFourVect(double vect[4], double a) const;
+
+  ///\brief add second vector to first one
+  void addFourVect(double u1[4], double const u2[4]) const;
+
+  ///\brief project u1 orthogonally to u2 at pos
+  void projectFourVect(double const pos[4], double u1[4], double const u2[4]) const;
+
+  /**
+   * \brief Computes dual 1-form
+   * Compute the dual 1-form of 4-vector.
+   * \param IN_ARRAY1_1 4-position;
+   * \param IN_ARRAY1_2 quadrivector;
+   * \param[out] ARGOUT_ARRAY3 output 1-form
+   */
+  void dualOneForm(double const IN_ARRAY1_1[4], double const IN_ARRAY1_2[4], double ARGOUT_ARRAY1[4]) const ;
 
   /**
    * \brief Computes the orthonormal local tetrad of the observer
-   * 
+   *
    * \param obskind  input: kind of observer (eg: "ZAMO","KeplerianObserver"...)
    * \param pos      input: position,
    * \param fourvel output: observer 4-velocity (norm -1)
@@ -385,11 +451,38 @@ class Gyoto::Metric::Generic
    * \param screen2 output: second vector in the screen plane
    * \param screen3 output: vector normal to the screen
    */
-  virtual void observerTetrad(std::string const obskind,
+  virtual void observerTetrad(obskind_t obskind,
 			      double const pos[4], double fourvel[4],
 			      double screen1[4], double screen2[4],
 			      double screen3[4]) const ;
 
+  /**
+   * \brief Computes the orthonormal local tetrad of the observer
+   *
+   * \param[in]  pos     position,
+   * \param[in]  fourvel observer 4-velocity (norm -1)
+   * \param[out] screen1 first vector in the screen plane
+   * \param[out] screen2 second vector in the screen plane
+   * \param[out] screen3 vector normal to the screen
+   */
+  virtual void observerTetrad(double const pos[4], double fourvel[4],
+			      double screen1[4], double screen2[4],
+			      double screen3[4]) const ;
+
+  /**
+   * \brief Apply Gram-Schmidt orthonormalization to a basis
+   *
+   * On input, u0 to u3 must be four non-zero norm, independent
+   * 4-vectors. On output, they will form an orthonormal basis.
+   *
+   * \param[in]     pos position,
+   * \param[in,out] u0 basis vector
+   * \param[in,out] u1 basis vector
+   * \param[in,out] u2 basis vector
+   * \param[in,out] u3 basis vector
+   */
+  void GramSchmidt(double const pos[4], double u0[4],
+		   double u1[4], double u2[4], double u3[4]) const;
   // Outputs
 
   /**
@@ -443,15 +536,23 @@ class Gyoto::Metric::Generic
   /**
    * \brief RK4 integrator
    */
-  virtual int myrk4(Worldline * line, const double coord[8], double h, double res[8]) const;
+  virtual int myrk4(Worldline * line, state_t const &coord, double h, state_t &res) const;
+  /// Obsolete, update your code
+  virtual int myrk4(Worldline * line, const double coord[8], double h, double res[8]) const = delete;
   
   /**
    * \brief RK4 integrator with adaptive step
    */
-  virtual int myrk4_adaptive(Gyoto::Worldline* line, const double coord[8],
+  virtual int myrk4_adaptive(Gyoto::Worldline* line, state_t const &coord,
 			     double lastnorm, double normref,
-			     double coordnew[8], double h0, double& h1,
+			     state_t &coordnew, double h0, double& h1,
 			     double deltamax=GYOTO_DEFAULT_DELTA_MAX) const;
+  /// Obsolete, update your code
+  virtual int myrk4_adaptive(Gyoto::Worldline* line, const double coord[8],
+                             double lastnorm, double normref,
+			     double coordnew[8], double h0, double& h1,
+                             double deltamax=GYOTO_DEFAULT_DELTA_MAX) const = delete;
+
 
   /**
    * \brief Check whether integration should stop
@@ -466,9 +567,11 @@ class Gyoto::Metric::Generic
   virtual int isStopCondition(double const coord[8]) const;
 
   /**
-   * \brief F function such as dy/dtau=F(y,cst)
+   * \brief F function such as dx/dt=F(x,cst)
    */
-  virtual int diff(const double y[8], double res[8]) const ;
+  virtual int diff(state_t const &x, state_t &dxdt) const ;
+  /// Obsolete, update your code
+  virtual int diff(const double y[8], double res[8]) const = delete ;
 
   /**
    * \brief Set Metric-specific constants of motion. Used e.g. in KerrBL.
