@@ -355,6 +355,12 @@ class Gyoto::Scenery
    */
   size_t getScalarQuantitiesCount(Quantity_t *q=NULL) const ;
 
+  /// Get number of requested quantities of spectral nature
+  /**
+   * This is Spectrum, SpectrumStokesQ, SpectrumStokesU, SpectrumStokesV and BinSpectrum.
+   */
+  size_t getSpectralQuantitiesCount(Quantity_t *q=NULL) const ;
+
   /// Get ph_.tmin_
   double tMin() const ;
   /// Get ph_.tmin_ in specified unit
@@ -400,6 +406,9 @@ class Gyoto::Scenery
   void secondary (bool sec) ; ///< Set ph_.secondary_
   bool secondary () const ; ///< Get ph_.secondary_
 
+  void parallelTransport (bool pt) ; ///< Set ph_.parallel_transport_
+  bool parallelTransport () const ; ///< Get ph_.parallel_transport_
+
   void maxiter (size_t miter) ; ///< Set ph_.maxiter_
   size_t maxiter () const ; ///< Get ph_.maxiter_
 
