@@ -5,7 +5,7 @@
  */
 
 /*
-    Copyright 2011-2017, 2019 Thibaut Paumard, Frederic Vincent
+    Copyright 2011-2019 Thibaut Paumard, Frederic Vincent
 
     This file is part of Gyoto.
 
@@ -239,7 +239,7 @@ class Gyoto::Screen
    * \brief What kind of observer are we considering? (At infinity, ZAMO...)
    *
    */
-  std::string observerkind_;
+  obskind_t observerkind_;
 
  public:
   GYOTO_OBJECT;
@@ -553,6 +553,18 @@ class Gyoto::Screen
    */
   void getRayCoord(double x, double y, double dest[8]) const;
 
+  /// Get polarization triad
+  /**
+   * Computes the polarization triad (Ephi,Etheta,k) from the
+   * tangent to photon geodesic k.
+   * \param[in] coord Photon coordinate + tangent vector (double[8])
+   * \param[out] Ephi first polarisation direction. Preallocated. Default: NULL.
+   * \param[out] Etheta second polarisation direction. Preallocated. Default: NULL.
+   * 
+   */
+  void getRayTriad(double coord[8],
+		   double Ephi[4], double Etheta[4]) const;
+
   /// Get 8-coordinate of Photon hitting screen pixel
   /**
    * Similar to Screen::getObserverPos() but will return in addition
@@ -741,11 +753,12 @@ class Gyoto::Screen
     /// 1D specifier for an arbitrary pixel coordinate set.
     class Indices : public Coord1dSet {
     protected:
-      size_t const * const indices_;
+      size_t * indices_;
       size_t const sz_;
       size_t i_;
     public:
       Indices (size_t const*const buf, size_t sz);
+      ~Indices();
       void begin();
       bool valid();
       size_t size();
@@ -757,11 +770,12 @@ class Gyoto::Screen
     /// 1D specifier for an arbitrary angle coordinate set.
     class Angles : public Coord1dSet {
     protected:
-      double const * const buf_;
+      double * buf_;
       size_t const sz_;
       size_t i_;
     public:
       Angles (double const*const buf, size_t sz);
+      ~Angles();
       void begin();
       bool valid();
       size_t size();
