@@ -1,5 +1,5 @@
 /*
-    Copyright 2013-2015 Frederic Vincent, Thibaut Paumard
+    Copyright 2013-2015, 2018 Frederic Vincent, Thibaut Paumard
 
     This file is part of Gyoto.
 
@@ -46,7 +46,7 @@ class Gyoto::Astrobj::ThinDiskIronLine
 
   using ThinDisk::emission;
   virtual double emission(double nu_em, double dsem,
-			  double c_ph[8], double c_obj[8]=NULL) const;
+			  state_t const &c_ph, double const c_obj[8]=NULL) const;
   void getVelocity(double const pos[4], double vel[4]);
 
 
