@@ -9,7 +9,7 @@
  */
 
 /*
-    Copyright 2011 Frederic Vincent, Thibaut Paumard
+    Copyright 2011-2014, 2018-2019 Frederic Vincent, Thibaut Paumard
 
     This file is part of Gyoto.
 
@@ -95,8 +95,6 @@ class Gyoto::Astrobj::UniformSphere :
  protected:
   double radius_ ; ///< sphere radius [geometrical units]
   bool isotropic_; ///< if 1, then emission just returns 1
-  double alpha_; ///< such that nu*I_nu = nu^alpha_; note that Xray photon
-              ///< index Gamma is: alpha_ = 2-Gamma
   SmartPointer<Spectrum::Generic> spectrum_; ///< sphere emission law
   SmartPointer<Spectrum::Generic> opacity_; ///< if optically thin, opacity law
 
@@ -188,16 +186,14 @@ class Gyoto::Astrobj::UniformSphere :
 
   using Standard::emission;
   virtual double emission(double nu_em, double dsem,
-			  double cp[8], double co[8]=NULL) const;
+			  state_t const &cp, double const co[8]=NULL) const;
   ///< Emission is determined by spectrum_ and opacity_
   using Standard::integrateEmission;
   virtual double integrateEmission(double nu1, double nu2, double dsem,
-				   double c_ph[8], double c_obj[8]=NULL) const;
-  virtual double transmission(double nuem, double dsem, double*) const ;
+				   state_t const &c_ph, double const *c_obj=NULL) const;
+  virtual double transmission(double nuem, double dsem, state_t const &, double const *) const ;
   ///< Transmission is determined by opacity_
-  virtual void processHitQuantities(Photon* ph, double* coord_ph_hit,
-				    double* coord_obj_hit, double dt,
-				    Astrobj::Properties* data) const;
+
 };
 
 
