@@ -4,7 +4,7 @@
  */
 
 /*
-    Copyright 2011-2016 Frederic Vincent, Thibaut Paumard
+    Copyright 2011-2018 Frederic Vincent, Thibaut Paumard
 
     This file is part of Gyoto.
 
@@ -28,6 +28,7 @@
 #include <iostream>
 #include <fstream>
 #include <string>
+#include <vector>
 
 #include <GyotoDefs.h>
 
@@ -58,6 +59,8 @@ namespace Gyoto {
   Gyoto::Property("Gyoto::Worldline", "Time-like or null geodesic."),	\
     GYOTO_PROPERTY_BOOL(c, HighOrderImages, PrimaryOnly, _secondary,	\
 			"Whether to stop Photon integration at 180° deflection.") \
+    GYOTO_PROPERTY_BOOL(c, ParallelTransport, NoParallelTransport, _parallelTransport,	\
+			"Whether to perform parallel transport of a local triad (used for polarization).") \
     GYOTO_PROPERTY_DOUBLE(c, RelTol, _relTol,				\
 			  "Relative tolerance for the adaptive step integrators.") \
     GYOTO_PROPERTY_DOUBLE(c, AbsTol, _absTol,				\
@@ -99,6 +102,8 @@ namespace Gyoto {
 #define GYOTO_WORLDLINE_ACCESSORS(c)					\
   void c::_secondary(bool s) {secondary(s);}				\
   bool c::_secondary() const {return secondary();}			\
+  void c::_parallelTransport(bool s) {parallelTransport(s);}		\
+  bool c::_parallelTransport() const {return parallelTransport();}	\
   void c::_adaptive(bool s) {adaptive(s);}				\
   bool c::_adaptive() const {return adaptive();}			\
   void c::_relTol(double f){relTol(f);}					\
@@ -161,6 +166,8 @@ namespace Gyoto {
   bool _adaptive () const ;				\
   void _secondary (bool sec) ;				\
   bool _secondary () const ;				\
+  void _parallelTransport (bool sec) ;			\
+  bool _parallelTransport () const ;			\
   void _maxiter (size_t miter) ;			\
   size_t _maxiter () const ;				\
   void _integrator(std::string const & type);		\
@@ -227,6 +234,7 @@ class Gyoto::Worldline
 
  protected:
   SmartPointer<Gyoto::Metric::Generic> metric_ ; ///< The Gyoto::Metric in this part of the universe
+  double* tau_; ///< proper time or affine parameter
   double* x0_;///< t or T
   double* x1_;///< r or x
   double* x2_;///< &theta; or y
@@ -235,6 +243,14 @@ class Gyoto::Worldline
   double* x1dot_;///< rdot or xdot
   double* x2dot_;///< &theta;dot or ydot
   double* x3dot_;///< &phi;dot or zdot
+  double* ep0_;/// Coordinate of first base vector to parallel transport
+  double* ep1_;/// Coordinate of first base vector to parallel transport
+  double* ep2_;/// Coordinate of first base vector to parallel transport
+  double* ep3_;/// Coordinate of first base vector to parallel transport
+  double* et0_;/// Coordinate of Second base vector to parallel transport
+  double* et1_;/// Coordinate of Second base vector to parallel transport
+  double* et2_;/// Coordinate of Second base vector to parallel transport
+  double* et3_;/// Coordinate of Second base vector to parallel transport
   size_t x_size_;///< Size of #x0_, #x1_... arrays
   size_t imin_;///< Minimum index for which #x0_, #x1_... have been computed
   size_t i0_;  ///< Index of initial condition in array
@@ -248,6 +264,14 @@ class Gyoto::Worldline
    */
   bool secondary_;
 
+  /**
+   * \brief Whether to parallel transport a local triad
+   *
+   * Typically used to trace the base in which the Stokes parameters
+   * are expressed in the context of polarization.
+   */
+  bool parallel_transport_;
+
   /**
    * \brief Initial integrating step
    *
@@ -374,7 +398,34 @@ class Gyoto::Worldline
    *
    * \param coord new initial coordinates
    * \param dir direction of integration. 1 for forward integration,
-   *        -1 for backards integration, 0 for unknown or both.
+   *        -1 for backards integration, 0 for unknown or both
+   * \param Ephi initial value of base vector to parallel-transport.
+   *        Ignored if #parallel_transport_ is false.
+   * \param Etheta initial value of base vector to parallel-transport.
+   *        Ignored if #parallel_transport_ is false.
+   */
+  virtual void   setInitCoord(const double coord[8], int dir,
+			      double const Ephi[4],
+			      double const Etheta[4]);
+
+
+  /**
+   * \brief Set Initial coordinate
+   *
+   * Set #imin_=#imax_=#i0_, and x<i>_[i0_]=coord[<i>].
+   *
+   * If dir==1, #i0_ is set to 0. If dir==-1, #i0_ is set to #x_size_-1.
+   *
+   * If dir==0 and the Worldine has never been computed (#i0_==0,
+   * #imin_==1 and #imax_==0), then dir defaults to 1 for a massive
+   * particle and -1 for a massless particle.
+   *
+   * If dir==0 and the Worldine has already been computed, #i0_ is not
+   * changed.
+   *
+   * \param coord new initial coordinates
+   * \param dir direction of integration. 1 for forward integration,
+   *        -1 for backards integration, 0 for unknown or both
    */
   virtual void   setInitCoord(const double coord[8], int dir = 0);
 
@@ -479,7 +530,6 @@ class Gyoto::Worldline
    * new index of old first element
    */
   virtual size_t xExpand(int dir); ///< Expand x0, x1 etc... to hold more elements
- 
 
   /**
    * If you need to expand more arrays than x0_ ... x3_ and the dots,
@@ -490,6 +540,21 @@ class Gyoto::Worldline
    */
   virtual void xExpand(double * &x, int dir); ///< Expand one array to hold more elements
 
+  /**
+   * Allocate memory for polarization vectors
+   */
+  virtual void eAllocate (); ///< Allocate ep0_ ... et3_.
+
+  /**
+   * Deallocate memory for polarization vectors
+   */
+  virtual void eDeallocate (); ///< Deallocate ep0_ ... et3_.
+
+  /**
+   * Call #xExpand(double * &x, int dir) on #ep0_, #ep1_ etc.
+   */
+  virtual void eExpand(int dir); /// Expand memory slots for polarization vectors
+
   // Mutators / assignment
   // ---------------------
  public:
@@ -506,6 +571,8 @@ class Gyoto::Worldline
   bool adaptive () const ; ///< Get #adaptive_
   void secondary (bool sec) ; ///< Set #secondary_
   bool secondary () const ; ///< Get #secondary_
+  void parallelTransport (bool pt) ; ///< Set #parallel_transport_
+  bool parallelTransport () const ; ///< Get #parallel_transport_
   void maxiter (size_t miter) ; ///< Set #maxiter_
   size_t maxiter () const ; ///< Get #maxiter_
 
@@ -529,18 +596,62 @@ class Gyoto::Worldline
    *        i.e. the 4-position and the 4-velocity of the Photon at
    *        the receiving end;
    * \param dir direction: 1 for future, -1 for past.
+   * \param Ephi initial value of base vector to parallel-transport.
+   *        Ignored if #parallel_transport_ is false.
+   * \param Etheta initial value of base vector to parallel-transport.
+   *        Ignored if #parallel_transport_ is false.
    */
   void setInitialCondition(SmartPointer<Metric::Generic> gg, 
+			   const double coord[8],
+			   const int dir,
+			   double const Ephi[4], double const Etheta[4]) ;
+
+  /// Set or re-set the initial condition prior to integration.
+  /**
+   * \param gg    Gyoto::SmartPointer to the Gyoto::Metric in this universe;
+   * \param coord 8 element array containing the initial condition,
+   *        i.e. the 4-position and the 4-velocity of the Photon at
+   *        the receiving end;
+   * \param dir direction: 1 for future, -1 for past.
+   */
+  void setInitialCondition(SmartPointer<Metric::Generic> gg,
 			   const double coord[8],
 			   const int dir) ;
 
-  void getInitialCoord(double dest[8]) const; ///< Get initial coordinate
-  void getCoord(size_t index, double dest[8]) const; ///< Get coordinates corresponding to index
+  /**
+   * Depending on the size of dest and on the value of
+   * #parallel_transport_, get position (xi_), velocity (xidot_) and
+   * possibly other triad vectors (epi_ and eti_).
+   */
+  void getInitialCoord(std::vector<double> &dest) const; ///< Get initial coordinates + base vectors
+
+  /**
+   * Depending on the value of #parallel_transport_, get position
+   * (xi_), velocity (xidot_) and possibly other triad vectors (epi_
+   * and eti_). coord is resized to the right number of elements.
+   */
+  void getCoord(size_t index, Gyoto::state_t &dest) const; ///< Get coordinates+base vectors corresponding to index
+
+  /**
+   * We need this non-const implementation to allow the const, size_t
+   * and the non-const, double implementations to coexist.
+   */
+  void getCoord(size_t index, Gyoto::state_t &dest) ; ///< Get coordinates+base vectors corresponding to index
+
+  /**
+   * Depending on the value of #parallel_transport_, get position
+   * (xi_), velocity (xidot_) and possibly other triad vectors (epi_
+   * and eti_). coord is resized to the right number of elements.
+   */
+  void getCoord(double date, Gyoto::state_t &dest, bool proper=false); ///< Get coordinates+base vectors corresponding to date dest[0].
+
   void getCartesianPos(size_t index, double dest[4]) const; ///< Get Cartesian expression of 4-position at index.
 
 
-  virtual void xStore(size_t ind, double const coord[8]) ; ///< Store coord at index ind
-  virtual void xFill(double tlim) ; ///< Fill x0, x1... by integrating the Worldline from previously set inittial condition to time tlim
+  virtual void xStore(size_t ind, state_t const &coord, double tau) ; ///< Store coord at index ind
+  virtual void xStore(size_t ind, state_t const &coord) =delete; ///< Obsolete, update your code
+  virtual void xStore(size_t ind, double const coord[8]) = delete; ///< Obsolete, update your code
+  virtual void xFill(double tlim, bool proper=false) ; ///< Fill x0, x1... by integrating the Worldline from previously set inittial condition to time tlim
 
 
 
@@ -557,6 +668,10 @@ class Gyoto::Worldline
    */
   void get_t(double *dest) const;
 
+  /**
+   * \brief Get computed proper times or values of the affine parameter
+   */
+  void get_tau(double *dest) const;
   
   /// Get the 6 Cartesian coordinates for specific dates.
   /**
@@ -595,20 +710,35 @@ class Gyoto::Worldline
    * dates. The line will be integrated further as required. An error
    * will be thrown if it is not possible to reach a certain date.
    *
-   * \param dates the list of dates for which the coordinates are to
-   *                be computed;
-   * \param n_dates the number of dates to compute ;
-   * \param x1dest, x2dest, x3dest, x0dot, x1dot, x2dot, x3dot arrays
+   * \param[in] dates the list of dates for which the coordinates are to
+   *                be computed in proper time or affine parameter if
+   *                #proper is true or in coordinate time if #proper
+   *                is false (default);
+   * \param[in] n_dates the number of dates to compute ;
+   * \param[out] x1dest, x2dest, x3dest, x0dot, x1dot, x2dot, x3dot arrays
    *               in which to store the result. These pointer may be
    *               set to NULL to retrieve only part of the
    *               information. They must be pre-allocated.
+   * \param[out] ephi0, ephi1, ephi2, ephi3, etheta0, etheta1,
+   *               etheta2, etheta3 arrays in which to store the ephi
+   *               and etheta (parallel transport case). These pointer
+   *               may be set to NULL to retrieve only part of the
+   *               information. They must be pre-allocated.
+   * \param[out] otime array in which to store the other time:
+   *               coordinate time if #proper, else proper time or
+   *               affine parameter.
+   * \param[in] proper bool: whether #dates is proper time (or affine
+   *               parameter) or coordinate time.
    *
    */
   void getCoord(double const * const dates, size_t const n_dates,
 		double * const x1dest,
 		double * const x2dest, double * const x3dest,
 		double * const x0dot=NULL,  double * const x1dot=NULL,
-		double * const x2dot=NULL,  double * const x3dot=NULL) ;
+		double * const x2dot=NULL,  double * const x3dot=NULL,
+		double * ep0=NULL, double * ep1=NULL, double * ep2=NULL, double * ep3=NULL,
+		double * et0=NULL, double * et1=NULL, double * et2=NULL, double * et3=NULL,
+		double * otime=NULL, bool proper=false) ;
 
   /**
    * \brief Get all computed positions
@@ -677,6 +807,7 @@ class Gyoto::Worldline
  */
 class Gyoto::Worldline::IntegState::Generic : public SmartPointee {
   friend class Gyoto::SmartPointer<Gyoto::Worldline::IntegState::Generic>;
+
  protected:
   /// Worldline that we are integrating.
   /**
@@ -686,6 +817,7 @@ class Gyoto::Worldline::IntegState::Generic : public SmartPointee {
   Worldline * line_;
   double delta_; ///< Integration step (current in case of #adaptive_).
   bool adaptive_; ///< Whether to use an adaptive step
+  bool parallel_transport_; ///< Whether to parallel-transport base vectors
   double norm_; ///< Current norm of the 4-velocity.
   double normref_; ///< Initial norm of the 4-velocity.
   /// The Metric in this end of the Universe.
@@ -720,7 +852,9 @@ class Gyoto::Worldline::IntegState::Generic : public SmartPointee {
    * \param coord Initial coordinate.
    * \param delta Integration step. Sign determines direction.
    */
-  virtual void init(Worldline * line, const double *coord, const double delta);
+  virtual void init(Worldline * line, const state_t &coord, const double delta);
+  /// Obsolete, update your code
+  virtual void init(Worldline * line, const double *coord, const double delta) = delete;
 
   /**
    * \brief Cache whatever needs to be cached
@@ -748,9 +882,12 @@ class Gyoto::Worldline::IntegState::Generic : public SmartPointee {
   /// Make one step.
   /**
    * \param[out] coord Next position-velocity;
+   * \param[out] tau   Next proper time or affine parameter
    * \param[in] h1max maximum step in case of adaptive integration
    */
-  virtual int nextStep(double *coord, double h1max=GYOTO_DEFAULT_DELTA_MAX)=0;
+  virtual int nextStep(state_t &coord, double &tau, double h1max=GYOTO_DEFAULT_DELTA_MAX)=0;
+  /// Obsolete, update your code
+  virtual int nextStep(double *coord, double h1max=GYOTO_DEFAULT_DELTA_MAX) = delete;
 
   /// Make one step of exactly this size.
   /**
@@ -762,14 +899,21 @@ class Gyoto::Worldline::IntegState::Generic : public SmartPointee {
    * \param[in] step  exact step to use.
    * \param[out] coordout next position-velocity;
    */
-  virtual void doStep(double const coordin[8], 
+  virtual void doStep(state_t const &coordin, 
 		      double step,
-		      double coordout[8])=0;
+		      state_t &coordout)=0;
+  /// Obsolete, update your code
+  virtual void doStep(double const coordin[8], 
+                      double step,
+		      double coordout[8]) = delete;
+
 };
 
 /**
  * \class Gyoto::Worldline::IntegState::Legacy
- * \brief Home-brewed integrator
+ * \brief Obsolete: Home-brewed integrator
+ *
+ * Will be removed soon.
  *
  * The integrator used by this IntegState::Generic implementation is
  * actually implemented in Metric::Generic::myrk4_adaptive(). It does
@@ -782,7 +926,7 @@ class Gyoto::Worldline::IntegState::Legacy : public Generic {
   friend class Gyoto::SmartPointer<Gyoto::Worldline::IntegState::Legacy>;
 
  private:
-  double coord_[8]; ///< Previously determined coordinate.
+  state_t coord_; ///< Previously determined coordinate.
 
  public:
   /// Constructor
@@ -790,14 +934,14 @@ class Gyoto::Worldline::IntegState::Legacy : public Generic {
   Legacy(Worldline *parent);
   Legacy * clone(Worldline*newparent) const ;
   using Generic::init;
-  void init(Worldline * line, const double *coord, const double delta);
+  void init(Worldline * line, const state_t &coord, const double delta);
   virtual std::string kind();
 
-  virtual int nextStep(double *coord, double h1max=1e6);
+  virtual int nextStep(state_t &coord, double &tau, double h1max=1e6);
 
-  virtual void doStep(double const coordin[8], 
+  virtual void doStep(state_t const &coordin, 
 		      double step,
-		      double coordout[8]);
+		      state_t &coordout);
 
   virtual ~Legacy();
 };
@@ -827,12 +971,19 @@ class Gyoto::Worldline::IntegState::Boost : public Generic {
   /// Integrator flavour
   Kind kind_;
 
+  typedef std::function<boost::numeric::odeint::controlled_step_result
+    (state_t&, double&, double&)> try_step_t;
+  typedef std::function<void(state_t&, double)> do_step_t;
+  typedef std::function<void(const state_t &/*x*/,
+			     state_t & /*dxdt*/,
+			     const double /* t*/ )> system_t;
+
   /// Stepper used by the adaptive-step integrator
-  std::function<boost::numeric::odeint::controlled_step_result
-    (std::array<double,8>&, double&, double&)> try_step_;
+  try_step_t try_step_;
 
   /// Stepper used by the non-adaptive-step integrator
-  std::function<void(std::array<double,8>&, double)> do_step_;
+  do_step_t do_step_;
+
  public:
   /// Constructor
   /**
@@ -852,11 +1003,11 @@ class Gyoto::Worldline::IntegState::Boost : public Generic {
   Boost * clone(Worldline* newparent) const ;
   virtual ~Boost();
   virtual void init();
-  virtual void init(Worldline * line, const double *coord, const double delta);
-  virtual int nextStep(double *coord, double h1max=1e6);
-  virtual void doStep(double const coordin[8], 
+  virtual void init(Worldline * line, const state_t &coord, const double delta);
+  virtual int nextStep(state_t &coord, double &tau, double h1max=1e6);
+  virtual void doStep(state_t const &coordin, 
 		      double step,
-		      double coordout[8]);
+		      state_t &coordout);
   virtual std::string kind();
   
 };
